/*
    Copyright (c) 2015 digitalSTROM AG, Zurich, Switzerland

    Authors: Christian Hitz, digitalSTROM AG <christian.hitz@digitalstrom.com>
             Michael Tross, digitalSTROM AG <michael.tross@digitalstrom.com>
             Reto Flütsch, digitalSTROM AG <reto.fluetsch@digitalstrom.com>

    This file is part of vdSM.

    vdSM is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 2 of the License.
    

    vdSM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vdSM. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SELECTB_H_
#define SELECTB_H_

#include "qp_port.h"
#include "types.h"
#include "dsm.h"
#include "vdc-api-proto.h"

typedef enum {
    UpstreamMainqualifierParameter0       = 0x0,
    UpstreamMainqualifierParameter1       = 0x1,
    UpstreamMainqualifierSensor           = 0x2,
    UpstreamMainqualifierDownstreamMirror = 0x3,
    UpstreamMainqualifierGiveUp           = 0xC,
    UpstreamMainqualifierStatisticValuesH = 0xD,
    UpstreamMainqualifierStatisticValuesL = 0xE,
    UpstreamMainqualifierDSID             = 0xF,
} UpstreamMainqualifier;


typedef enum {
    UpstreamSubqualifierSensorSubSub       = 0x00,
    UpstreamSubqualifierSensorSensorList   = 0x01,
    UpstreamSubqualifierSensorSensorValues = 0x02,
    UpstreamSubqualifierBinaryInputStates  = 0x03,
    UpstreamSubqualifierSensorStatusStart  = 0x10,
    UpstreamSubqualifierSensorStatusEnd    = 0x1F,
} UpstreamSubqualifierSensor;

typedef enum {
    UpstreamSubsubqualifierTestResponse = 0x00,
    UpstreamSubsubqualifierUDIResponse  = 0x01,
} UpstreamSubsubqualifier;


typedef struct {
    uint8_t requestAddress;
    uint8_t transactionId;
    uint8_t messageId;
    uint8_t modifier;
} SourceInfo;

/**
 * Get device statistics event structure
 */
typedef struct {
    QEvent super;
    uint16_t deviceId;
    uint8_t paramOffset;
} GetDeviceStatisticsEvt;

/**
 * Get device sensor event structure
 */
typedef struct {
    QEvent super;
    uint16_t deviceId;
    uint8_t sensorId;
    uint8_t padding;
} GetDeviceSensorEvt;

typedef enum {
    DSLINK_BYTE_NO_ANSWER = 0,
    DSLINK_LAST_BYTE_NO_ANSWER,
    DSLINK_BYTE_WITH_ANSWER,
    DSLINK_LAST_BYTE_WITH_ANSWER,
} DSLinkOperation_t;

/**
 * Get device sensor event structure
 */
typedef struct {
    QEvent super;
    uint16_t deviceId;
    DSLinkOperation_t operation;
    uint8_t value;
} GetDevicedSLinkEvt;

/**
 * Get zone/group sensor event structure
 */
typedef struct {
    QEvent super;
    DSLinkOperation_t operation;
    uint8_t localZoneId;
    uint8_t groupId;
    uint8_t value;
} GetZoneGroupdSLinkEvt;

/**
 * Device parameter event structure
 */
typedef struct {
    QEvent super;
    uint32_t deviceSerial;
    uint16_t deviceId;
    uint8_t paramBank;
    uint8_t paramOffset;
    uint8_t value;
    uint8_t sendDoneModifier;
} DeviceParameterEvt;

/**
 * Device parameter event structure
 */
typedef struct {
    QEvent super;
    uint32_t deviceSerial;
    uint16_t deviceId;
    uint16_t value;
    uint8_t channel;
    uint8_t scene;
    uint8_t operation;
    bool is16bit;
} DeviceOutpuChannelEvt;

/**
 * Get device transmission quality
 */
typedef struct {
    QEvent super;
    uint16_t deviceId;
} GetTransmissionQualityEvt;

/**
 * Get device presence
 */
typedef struct {
    QEvent super;
    uint16_t deviceId;
} GetDevicePresenceEvt;

/**
 * Disable/enable parts of the SelectB HSM event structure
 */
typedef struct {
    QEvent super;
    bool disable;
} DisableSelectBPartsEvt;

/**
 * Configure ping event structure
 */
typedef struct {
    QEvent super;
    uint8_t modifier;
    uint16_t oneTimeInterval;
    uint16_t autoInterval;
} ConfigurePingEvt;

typedef struct {
    QEvent super;
    uint16_t deviceId;
} PingInactiveDeviceEvt;

/**
 * Select Device in B
 */
typedef struct {
    QEvent super;
    uint16_t deviceId;
} SelectDeviceEvt;

typedef struct {
    QEvent super;
    uint16_t deviceId;
} PingResponseEvt;

/**
 * SelectB user protobuf message request event structure
 */
typedef struct {
    QEvent super;
    dsuid_t dsuid;
    uint16_t size;
    uint8_t request[VDC_USER_PROTOBUF_MESSAGE_LENGTH];
} SelectBProtobufMessageRequestEvt;

/**
 * SelectB user protobuf message response event structure
 */
typedef struct {
    QEvent super;
    uint16_t size;
    uint8_t response[VDC_USER_PROTOBUF_MESSAGE_LENGTH];
} SelectBProtobufMessageResponseEvt;

void SelectB_ctor(void);

extern QActive * const AO_SelectB; /* "opaque" pointer to Select B AO */

/**
 * Return margin of deferred command event queue
 */
uint8_t SelectB_GetDeferredCommandQueueMargin(void);

/**
 * Return margin of deferred dSID event queue
 */
uint8_t SelectB_GetDeferredDsidQueueMargin(void);

bool SelectB_CheckForCrosstalk(LongUpstreamEvt const * longUpEvt);
void SelectB_sendRestartRegistrationSignal();

uint32_t SelectB_GetTimeSeconds();

void SelectB_AnnounceDevice(const char* dsuid, const char *vdc_dsuid, uint32_t message_id);
void SelectB_UserProtobufMessageResponse(uint16_t size, const uint8_t* response);

#endif /* SELECTB_H_ */
